/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2017 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"

#include "stdio.h"

#if defined( ENABLE_SYS_LOG )       //K{ 
  #include "sys_log/sys_log.h"
  #include "sys_log/sys_log_define.h"
#endif

#include "audio.h"
extern AUDIO_DATA audioData;





int delay_Clock = 200000000;   //VXeNbNF200MHz
char Buf[64];


void delay_us(volatile unsigned int usec)        //1secx
{
        volatile  int count;

        count = (int)(delay_Clock/20000000)*usec;
        do      //@at@200MH (Clock=200000000)
        {       //delay_us(1000)F1000.4sec  delay_us(100)F100.6sec  delay_us(10):10.5sec@@delay_us(1)F1.5sec
                asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP");asm("NOP");
                asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP");

                count--;
        }while(count != 0);
}



void delay_ms(volatile unsigned int msec)        //1msecx
{
        volatile unsigned int i;         //Fat200MH (Clock=200000000)//delay_ms(1)F 1.0006msec   delay_ms(100)F100.04msec

        for(i=0; i<msec; i++)
        delay_us(1000);
}


// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize()

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize()
{
    APP_LED1_ON();      // all LED's on during initialization    
    APP_LED2_ON();
    APP_LED3_ON();
    APP_LED4_ON();
    APP_LED5_ON();
    
    
    ysFunc123();
    //LATCbits.LATC14 = 1;
            
    lcd_init();				// LCD
    lcd_cmd(0b00001100);    // J[\:OFF  uݸ:OFF
    lcd_clear(); 
    lcd_cmd(0x80);          //1ڂ̐擪
    sprintf(Buf,"BM64 A2DP audio          ");//
    lcd_str(Buf);                   //t\
    lcd_cmd(0xC0);          //Qsڂ̐擪
    sprintf(Buf,"  Android Music             ");	//
    lcd_str(Buf);           // JnbZ[W1sڕ\
    
    delay_ms(3000);
    
       
    //Place the App state machine in its initial state.
    appData.state = APP_STATE_INIT;
    
    appData.volume = 0x40;
    
    appData.buttonState = BUTTON_STATE_IDLE;
    appData.buttonDelay = 0;        // used for button debounce
    appData.volumeDelay = 0;         // used for volume widget
    appData.linkStatus = -1;        // force mismatch 
    
    audioInitialize();
    
    
    REFO3CONbits.ON = 0;    //Reference Oscillator module is disabled
    REFO3CONbits.ROSEL = 0b0010;    //POSC  //唭UgF24Mz  //RO: Referece Oscillator
    REFO3CONbits.RODIV = 5;         //N = 5 Reference3 o͎gF Fposc @(2 x (N + M/512))
    REFO3TRIM = 440;                //M = 440 
                                    //Reference3 o͎gF24MHz @(2 x (5 + 440/512)) = 2.048MHz
                                    //lF2.4005MHz(why ?)
    REFO3CONbits.ON = 1;    //Reference Oscillator module is Enabled
    REFO3CONbits.OE = 1;    //Reference clock is driven out on REFCLKO3 pin
    
    
    
    //PDN Zbg
    delay_ms(500);
    LATBbits.LATB4 = 1;
    
    delay_ms(500);
    LATBbits.LATB4 = 0;
            
     delay_ms(500);
    LATBbits.LATB4 = 1;
    
   
    
    
    
    
    
    
    
    APP_LED1_OFF();      // all LED's off
    APP_LED2_OFF();
    APP_LED3_OFF();
    APP_LED4_OFF();
    APP_LED5_OFF();    
 
 #if defined( ENABLE_SYS_LOG )      //K{ 
    SYS_LOG_Init();
    SYS_LOG("----------------------------------------");
    SYS_LOG("- Starting:");
    SYS_LOG("----------------------------------------");
  #endif

 

}  //End APP_Initialize()

/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */
SYS_TMR_HANDLE tmrHandle;
uint16_t lastappData_state = 0xffff;

void APP_Tasks()
{    
#if defined( ENABLE_SYS_LOG )
    if (appData.state != lastappData_state)    
    {
        SYS_LOG1("APP_Tasks: appData.state=%d",appData.state);
        lastappData_state = appData.state;
    }
#endif
  
    audioTasks(); 
   
    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
////ys            
//            laWidget_SetVisible((laWidget*)GFX_CONNECTED, LA_FALSE);
//            laWidget_SetVisible((laWidget*)GFX_PAIRED, LA_FALSE);
//            laWidget_SetVisible((laWidget*)GFX_NOPAIR_NOCONNECTION, LA_TRUE);
            
            tmrHandle = SYS_TMR_CallbackPeriodic (1, 0, &Timer_1ms);    //1msecɃR[obN֐Timer_1ms()ĂяoJn
            if (tmrHandle == DRV_HANDLE_INVALID)
            {
               
#if defined( ENABLE_SYS_LOG )    
                SYS_LOG("----------------------------------------");
                SYS_LOG("- Invalid tmrHandle in app.c");
                SYS_LOG("----------------------------------------");
#endif 
                // sticks in this state if handle invalid
            }
            else
            {           
                appData.state = APP_STATE_WAIT_INIT;
            }       
            break;
        }            

        case APP_STATE_WAIT_INIT:
        {
                
   
            if (DRV_BT_STATUS_READY == DRV_BT_GetPowerStatus())     //BM64̓dL̏ꍇ
            {  
                appData.state=APP_STATE_IDLE;
                audioStart();
            }
            break;
        }
        
        // Initialized 
        case APP_STATE_IDLE:
        {
            
                
            buttonTasks();
         
            
            break;
        }        
    }
 
    if ((audioData.bt.handle != (DRV_HANDLE)NULL) &&    //Bluetooth AudiõnhLȏꍇ
       (audioData.bt.handle != DRV_HANDLE_INVALID))
    {     
        DRV_BT_LINKSTATUS newLinkStatus = DRV_BT_GetLinkStatus(audioData.bt.handle);          
        // update connection status
        if (newLinkStatus & (DRV_BT_HFP_LINK_STATUS|DRV_BT_SCO_LINK_STATUS))
        {
          
            APP_LED4_ON();
            
        }
        else
        {
            //LED_
            APP_LED4_OFF();
        }
       
        
        if (newLinkStatus & DRV_BT_A2DP_LINK_STATUS)
        {
          
            APP_LED5_ON();
        }
        else
        {
              //LED_
            APP_LED5_OFF();
        }
        
       
        
        if (newLinkStatus != appData.linkStatus)
        {  
            if ((DRV_BT_GetPowerStatus() == DRV_BT_STATUS_READY))
            {
                if (newLinkStatus == DRV_BT_NO_LINK_STATUS)
                {    
////ys                    laString tempStr;                                      
                    char buf [DRV_BT_MAXBDNAMESIZE+1];
                    
////ys                
//                    laWidget_SetVisible((laWidget*)GFX_VOLUME_VALUE, LA_FALSE);                   
//                    laWidget_SetVisible((laWidget*)GFX_BTNAME, LA_TRUE);
//                    laWidget_SetVisible((laWidget*)GFX_BTADDRESS, LA_TRUE);

                    DRV_BT_GetBDName(audioData.bt.handle, buf, DRV_BT_MAXBDNAMESIZE+1);
                    
////ys                    
//                    tempStr = laString_CreateFromCharBuffer(buf, &LiberationSans12);
//                    laLabelWidget_SetText(GFX_BTNAME_VALUE, tempStr);
//                    laString_Destroy(&tempStr);

                    DRV_BT_GetBDAddress(audioData.bt.handle, buf);
                    
////ys                    tempStr = laString_CreateFromCharBuffer(buf, &LiberationSans12);
//                    laLabelWidget_SetText(GFX_BTADDRESS_VALUE, tempStr);
//                    laString_Destroy(&tempStr);
//                    
//                    laWidget_SetVisible((laWidget*)GFX_CONNECTED, LA_FALSE);
//                    laWidget_SetVisible((laWidget*)GFX_PAIRED, LA_FALSE);
//                    laWidget_SetVisible((laWidget*)GFX_NOPAIR_NOCONNECTION, LA_TRUE);                    
                }
                else
                {
                    
////ys                    laString tempStr;
//                    
//                    laWidget_SetVisible((laWidget*)GFX_BTNAME, LA_FALSE);
//                    laWidget_SetVisible((laWidget*)GFX_BTADDRESS, LA_FALSE);
//
//                    laWidget_SetVisible((laWidget*)GFX_AUDIO_MUTE, LA_FALSE);
//                    laWidget_SetVisible((laWidget*)GFX_AUDIO_PLAY, LA_FALSE);                    
//
//                    tempStr = laString_CreateFromCharBuffer(" ", &LiberationSans12);
//                    laLabelWidget_SetText(GFX_BTNAME_VALUE, tempStr);
//                    laLabelWidget_SetText(GFX_BTADDRESS_VALUE, tempStr);
//                    laString_Destroy(&tempStr);
//                    
//                    laWidget_SetVisible((laWidget*)GFX_CONNECTED, LA_TRUE);
//                    laWidget_SetVisible((laWidget*)GFX_PAIRED, LA_FALSE);
//                    laWidget_SetVisible((laWidget*)GFX_NOPAIR_NOCONNECTION, LA_FALSE);                    
                }

                appData.linkStatus = newLinkStatus;
            }
        }
    }
}
     
void buttonTasks( )
{   
    //BUTTON PROCESSING
    /* Check the buttons' current state. */      
    switch ( appData.buttonState )
    {
        case BUTTON_STATE_IDLE:
        {
            if ( (appData.buttonDelay==0)&&
                 (
                    Volume_Up_SwStateGet() == 0 ||  //RF13(SW4) //  Abv  //AK4384̂ݗLAPCM5102Ał͖
                    
                                            //PORTFbits.RF13 == 0 ||  //RF13(SW4) //  Abv
                                            //(BSP_SwitchStateGet(BSP_SWITCH_1)== 0||   //SW1  Volume Up (hold > 1 second for Begin Pairing function).
                    
                    Volume_Down_SwStateGet() == 0 ||  //RB13(SW5) //ʃ_E  //AK4384̂ݗLAPCM5102Ał͖
                                            //PORTBbits.RB15 == 0 ||  //RB13(SW5) //ʃ_E
                                            //(BSP_SwitchStateGet(BSP_SWITCH_2)== 0||   //SW2  Volume Down (hold > 1 second for Disconnect or Link Last function).  
                    
                    Next_Song_SwStateGet() == 0 ||  //RG15(SW3) //
                                            //PORTGbits.RG15 == 0 ||  //RG15(SW3) //
                                            //(BSP_SwitchStateGet(BSP_SWITCH_3)== 0||   //SW3  Play Next Song (hold > 1 second for Fast Forward). 
                    
                    Pause_Play_SwStateGet() == 0 ||  //RD15(SW9) //ꎞ~/ĊJ
                                            //PORTDbits.RD15 == 0 ||  //RD15(SW9) //ꎞ~/ĊJ
                                            //(BSP_SwitchStateGet(BSP_SWITCH_4)== 0||   //SW4  Play Pause Song
                    
                    Previou_Song_SwStateGet() == 0) )   //RA4(SW1)  //O 
                                            //PORTAbits.RA4 == 0) )   //RA4(SW1)  //O
                                            //(BSP_SwitchStateGet(BSP_SWITCH_5)== 0) )  //SW5  Play Previous Song (hold > 1 second for Rewind).  
            {
                appData.buttonDelay=BUTTON_DEBOUNCE;       
                appData.buttonState=BUTTON_STATE_PRESSED;               
            }
            
            break;
        }
    
        case BUTTON_STATE_PRESSED:
        { 
            if (appData.buttonDelay>0)
            {
                break;      // still debouncing
            }
            
            if(Volume_Up_SwStateGet() == 0)
                    //if(PORTFbits.RF13 == 0)
                    //if(BSP_SwitchStateGet(BSP_SWITCH_1)== 0)       // SW1 volume up/pairing
            {                
                appData.buttonDelay=LONG_BUTTON_PRESS;          // 1 sec is long press
                appData.buttonState=BUTTON_STATE_UPPAIRING_PRESSED;                  
            }
            else if(Volume_Down_SwStateGet() == 0)  // SW2 volume down
                    //else if(PORTBbits.RB15 == 0 )         // SW2 volume down 
                    //else if(BSP_SwitchStateGet(BSP_SWITCH_2)== 0)        // SW2 volume down
            {
                appData.buttonDelay=LONG_BUTTON_PRESS;          // 1 sec is long press
                appData.buttonState=BUTTON_STATE_DOWNCONNECT_PRESSED;                 
            }
            else if(Next_Song_SwStateGet() == 0)        // SW3 -- next song)
                    //else if (PORTGbits.RG15 == 0)    
                    //else if (BSP_SwitchStateGet(BSP_SWITCH_3)== 0)        // SW3 -- next song)
            {
                DRV_BT_PLAYINGSTATUS playingStatus = DRV_BT_GetPlayingStatus(audioData.bt.handle);
                if ((playingStatus==DRV_BT_PLAYING_FF)||(playingStatus==DRV_BT_PLAYING_FR))
                {
                    if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                    {                 
                        DRV_BT_CancelForwardOrRewind(audioData.bt.handle);
                    }
////ys                    
//                    laWidget_SetVisible((laWidget*)GFX_NEXT, LA_FALSE);                  
//                    laWidget_SetVisible((laWidget*)GFX_FASTFORWARD, LA_FALSE);
                    appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;
                }
                else
                {
                    appData.buttonDelay=LONG_BUTTON_PRESS;          // 1 sec is long press
                    appData.buttonState=BUTTON_STATE_NEXTFF_PRESSED;
////ys                    
//                    laWidget_SetVisible((laWidget*)GFX_NEXT, LA_TRUE);                 
                }
            }
            
             else if (Pause_Play_SwStateGet() == 0)       //SW4 -- play/pause 
                    //else if (PORTDbits.RD15 == 0) //SW4 -- play/pause
                    //else if (BSP_SwitchStateGet(BSP_SWITCH_4)== 0)       // SW4 -- play/pause
            {
                DRV_BT_PLAYINGSTATUS playingStatus = DRV_BT_GetPlayingStatus(audioData.bt.handle);
                if ((playingStatus==DRV_BT_PLAYING_FF)||(playingStatus==DRV_BT_PLAYING_FR))
                {
                    if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                    {                 
                        DRV_BT_CancelForwardOrRewind(audioData.bt.handle);
                    }
////ys                    
//                    laWidget_SetVisible((laWidget*)GFX_NEXT, LA_FALSE);                  
//                    laWidget_SetVisible((laWidget*)GFX_FASTFORWARD, LA_FALSE);
                    appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;
                }
                else
                {
                    appData.buttonDelay=3*LONG_BUTTON_PRESS;          // 3 sec is long press
                    appData.buttonState=BUTTON_STATE_PLAYPAUSE_PRESSED;                   
                }
            }
            else if (Previou_Song_SwStateGet() == 0)       // SW5 -- previous song
                    //else if (PORTAbits.RA4 == 0)          // SW5 -- previous song
                    //else if (BSP_SwitchStateGet(BSP_SWITCH_5)== 0)       // SW5 -- previous song
            {
                DRV_BT_PLAYINGSTATUS playingStatus = DRV_BT_GetPlayingStatus(audioData.bt.handle);
                if ((playingStatus==DRV_BT_PLAYING_FF)||(playingStatus==DRV_BT_PLAYING_FR))
                {
                    if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                    {                 
                        DRV_BT_CancelForwardOrRewind(audioData.bt.handle);
                    }
////ys                   
//                    laWidget_SetVisible((laWidget*)GFX_NEXT, LA_FALSE);                  
//                    laWidget_SetVisible((laWidget*)GFX_FASTFORWARD, LA_FALSE);
                    appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;
                }
                else
                {               
                    appData.buttonDelay=LONG_BUTTON_PRESS;          // 1 sec is long press
                    appData.buttonState=BUTTON_STATE_PREVREWIND_PRESSED;
////ys                    
//                    laWidget_SetVisible((laWidget*)GFX_PREVIOUS, LA_TRUE);
                }
            }             
            else
            {
                appData.buttonState=BUTTON_STATE_IDLE;                 // button not pressed anymore, assume noise                
            }

            break;
        }
        
        case BUTTON_STATE_UPPAIRING_PRESSED:
        {
            if ((appData.buttonDelay>0)&&
                  (Volume_Up_SwStateGet() != 0))    // SW1 pressed and released < 1 sec -- Next  
                        //(PORTFbits.RF13 != 0))     // SW1 pressed and released < 1 sec -- Next    
                        //(BSP_SwitchStateGet(BSP_SWITCH_1)!= 0))     // SW1 pressed and released < 1 sec -- Next
            {
                if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                {
                    DRV_BT_volumeUp(audioData.bt.handle);
                        //DRV_BT_volumeUpCurrentMode(audioData.bt.handle);
                        //DRV_BT_setVolCurrentMode();                
                }
                appData.buttonDelay=BUTTON_DEBOUNCE;                
                appData.buttonState=BUTTON_STATE_IDLE;              
            }
            else if ((appData.buttonDelay==0)&&
                    (Volume_Up_SwStateGet() == 0)) // SW1 still pressed after 1 sec -- fast forwar
                        //(PORTFbits.RF13== 0))       // SW1 still pressed after 1 sec -- fast forward
                        //(BSP_SwitchStateGet(BSP_SWITCH_1)== 0))       // SW1 still pressed after 1 sec -- fast forward
            {
                DRV_BT_EnterBTPairingMode(audioData.bt.handle);               
                appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;                
            }           
            break;
        } 

        case BUTTON_STATE_DOWNCONNECT_PRESSED:
        {
            if ((appData.buttonDelay>0)&&
                (Volume_Down_SwStateGet() != 0))  // SW2 pressed and released < 1 sec -- Next 
                        //(PORTBbits.RB15 != 0))     // SW2 pressed and released < 1 sec -- Next   
                        //(BSP_SwitchStateGet(BSP_SWITCH_2)!= 0))     // SW2 pressed and released < 1 sec -- Next
            {
                if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                {                
                    DRV_BT_volumeDown(audioData.bt.handle);
                    //DRV_BT_volumeDownCurrentMode(audioData.bt.handle);
                    //DRV_BT_setVolCurrentMode();
                }
                appData.buttonDelay=BUTTON_DEBOUNCE;                
                appData.buttonState=BUTTON_STATE_IDLE;              
            }
            else if ((appData.buttonDelay==0)&&
                    (Volume_Down_SwStateGet() == 0))    // SW2 still pressed after 1 sec -- fast forward
                            //(PORTBbits.RB15== 0))       // SW2 still pressed after 1 sec -- fast forward
                            // (BSP_SwitchStateGet(BSP_SWITCH_2)== 0))       // SW2 still pressed after 1 sec -- fast forward
            {
                if (DRV_BT_GetLinkStatus(audioData.bt.handle)!=0)
                {                 
                    DRV_BT_DisconnectAllLinks(audioData.bt.handle);
                }
                else
                {                 
                    DRV_BT_LinkLastDevice(audioData.bt.handle);
                }                
                appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;                
            }
            
            break;
        } 
        
        case BUTTON_STATE_NEXTFF_PRESSED:
        {
            if ((appData.buttonDelay>0)&&
                (Next_Song_SwStateGet() != 0))     // SW3 pressed and released < 1 sec -- Next
                        //(PORTGbits.RG15 != 0))     // SW3 pressed and released < 1 sec -- Next
                        //(BSP_SwitchStateGet(BSP_SWITCH_3)!= 0))     // SW3 pressed and released < 1 sec -- Next
            {
                if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                {                 
                    DRV_BT_PlayNextSong(audioData.bt.handle);
                }
////ys                
//                laWidget_SetVisible((laWidget*)GFX_NEXT, LA_FALSE);                  
//                laWidget_SetVisible((laWidget*)GFX_FASTFORWARD, LA_FALSE);
                appData.buttonDelay=BUTTON_DEBOUNCE;                
                appData.buttonState=BUTTON_STATE_IDLE;              
            }
            else if ((appData.buttonDelay==0)&&
                    (Next_Song_SwStateGet() == 0))       // SW3 still pressed after 1 sec -- fast forward
                    // (PORTGbits.RG15 == 0))       // SW3 still pressed after 1 sec -- fast forward
                    // (BSP_SwitchStateGet(BSP_SWITCH_3)== 0))       // SW3 still pressed after 1 sec -- fast forward
            {
                if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)                
                {                 
                    DRV_BT_FastForward(audioData.bt.handle);
                }
////ys                
//                laWidget_SetVisible((laWidget*)GFX_NEXT, LA_FALSE);  
//                laWidget_SetVisible((laWidget*)GFX_FASTFORWARD, LA_TRUE);                
                appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;                
            }
            
            break;
        }                      
        
        case BUTTON_STATE_PLAYPAUSE_PRESSED:
        {
            if ((appData.buttonDelay>0)&&
                (Pause_Play_SwStateGet() != 0))     // SW3 pressed and released < 1 sec -- previous   
                        //(PORTDbits.RD15 != 0))     // SW3 pressed and released < 1 sec -- previous
                        //(BSP_SwitchStateGet(BSP_SWITCH_4)!= 0))     // SW4 pressed and released < 1 sec -- play/pause
            {
                DRV_BT_PLAYINGSTATUS playingStatus = DRV_BT_GetPlayingStatus(audioData.bt.handle);
                if ((playingStatus==DRV_BT_PLAYING_FF)||(playingStatus==DRV_BT_PLAYING_FR))
                {
                    if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                    {                 
                        DRV_BT_CancelForwardOrRewind(audioData.bt.handle);
                    }                                       
                }
                else
                {
                    if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                    {                                                             
                        if (playingStatus==DRV_BT_PLAYING_PLAYING)                       
                        {
////ys                            
//                            laWidget_SetVisible((laWidget*)GFX_AUDIO_PLAY, LA_FALSE);     
//                            laWidget_SetVisible((laWidget*)GFX_AUDIO_MUTE, LA_TRUE);
                        }
                        else
                        {
////ys                            
//                            laWidget_SetVisible((laWidget*)GFX_AUDIO_PLAY, LA_TRUE);     
//                            laWidget_SetVisible((laWidget*)GFX_AUDIO_MUTE, LA_FALSE);
                        }                           
                        DRV_BT_PlayPause(audioData.bt.handle);                            
                    }
                }
                
////ys                
//                laWidget_SetVisible((laWidget*)GFX_NEXT, LA_FALSE);                  
//                laWidget_SetVisible((laWidget*)GFX_FASTFORWARD, LA_FALSE);
//                laWidget_SetVisible((laWidget*)GFX_PREVIOUS, LA_FALSE);                  
//                laWidget_SetVisible((laWidget*)GFX_REWIND, LA_FALSE);                
                appData.buttonDelay=BUTTON_DEBOUNCE;                
                appData.buttonState=BUTTON_STATE_IDLE;              
            }
            else if ((appData.buttonDelay==0)&&
                     (Previou_Song_SwStateGet() == 0))       // SW4 still pressed after 1 sec -- disconnect
                            //(PORTAbits.RA4 == 0))       // SW4 still pressed after 1 sec -- disconnect
                            //(BSP_SwitchStateGet(BSP_SWITCH_4)== 0))       // SW4 still pressed after 1 sec -- disconnect
            {
                if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                {                 
                    DRV_BT_ForgetAllLinks(audioData.bt.handle);               
                    appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE; 
                }                                                    
           
                appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;                
            }
            
            break;
        }

        case BUTTON_STATE_PREVREWIND_PRESSED:
        {
            if ((appData.buttonDelay>0)&&
                (Previou_Song_SwStateGet() != 0))   
                        //(PORTAbits.RA4 != 0))
                        //(BSP_SwitchStateGet(BSP_SWITCH_5)!= 0))     // SW3 pressed and released < 1 sec -- previous
            {
                if (DRV_BT_GetLinkStatus(audioData.bt.handle) & DRV_BT_AVRCP_LINK_STATUS)
                {                 
                    DRV_BT_PlayPreviousSong(audioData.bt.handle);
                }
////ys                
//                laWidget_SetVisible((laWidget*)GFX_PREVIOUS, LA_FALSE);                  
//                laWidget_SetVisible((laWidget*)GFX_REWIND, LA_FALSE);                 
                appData.buttonDelay=BUTTON_DEBOUNCE;                
                appData.buttonState=BUTTON_STATE_IDLE;              
            }
            else if ((appData.buttonDelay==0)&&
                    (Previou_Song_SwStateGet() == 0))       // SW3 still pressed after 1 sec -- fast forward
                            //(PORTAbits.RA4 != 0))       // SW3 still pressed after 1 sec -- fast forward
                            // (BSP_SwitchStateGet(BSP_SWITCH_5)== 0))       // SW3 still pressed after 1 sec -- fast forward
            {
                DRV_BT_Rewind(audioData.bt.handle);
////ys                
//                laWidget_SetVisible((laWidget*)GFX_PREVIOUS, LA_FALSE);
//                laWidget_SetVisible((laWidget*)GFX_REWIND, LA_TRUE);                
                appData.buttonState=BUTTON_STATE_WAIT_FOR_RELEASE;                
            }
            
            break;
        }  

        case BUTTON_STATE_WAIT_FOR_RELEASE:
        {
            if (
                 (Volume_Up_SwStateGet() != 0)&& 
                    //(PORTFbits.RF13 != 0)&&   
                    //(BSP_SwitchStateGet(BSP_SWITCH_1)!= 0)&&
               
                 (Volume_Down_SwStateGet() != 0)&&  
                    //(PORTBbits.RB15 != 0)&&   
                    //(BSP_SwitchStateGet(BSP_SWITCH_2)!= 0)&&
                 
                (Next_Song_SwStateGet() != 0)&&  
                    //(PORTGbits.RG15 != 0)&&
                    //(BSP_SwitchStateGet(BSP_SWITCH_3)!= 0)&&    
               
                    
                (Pause_Play_SwStateGet() != 0)&&  
                    //(PORTDbits.RD15 != 0)&&
                    //(BSP_SwitchStateGet(BSP_SWITCH_4)!= 0)&&    
                
                (Previou_Song_SwStateGet() != 0) )       
                    //(PORTAbits.RA4 != 0) )
                    //(BSP_SwitchStateGet(BSP_SWITCH_5)!= 0))
            {
                appData.buttonDelay=BUTTON_DEBOUNCE;
                appData.buttonState=BUTTON_STATE_IDLE;
            }
        }

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
    
}

void Timer_1ms(uintptr_t context, uint32_t currTick)
{
    //Button Processing
    if(appData.buttonDelay)
        --appData.buttonDelay;
    
    if(appData.volumeDelay)
    {
        --appData.volumeDelay;    
        if (appData.volumeDelay==0)
        {
////ys            
//            laWidget_SetVisible((laWidget*)GFX_VOLUME_VALUE, LA_FALSE);                          
        }
    }
}

uint32_t coretime_i = 0;
uint32_t coretime_f = 0;
uint32_t coretime [MAX_CORETIME_I];
uint32_t __attribute__((nomips16)) APP_ReadCoreTimer(void)
{
    uint32_t timer;
    // get the count reg
    asm volatile("mfc0   %0, $9" : "=r"(timer));
    return (timer);
}
//ys
//void APP_SaveCoreTime(void) //gĂȂ
//{
//    // to take measurement
//    if (coretime_f && (coretime_i < (MAX_CORETIME_I-1)))
//    {
//        coretime [coretime_i++] = APP_ReadCoreTimer();
//    }
//}

//ys
//void APP_StartCoreTime()    //ĂȂ
//{
//    coretime_f = 1;
//}
/*******************************************************************************
 End of File
 */
