/*******************************************************************************
  MPLAB Harmony Application Header File

  Company:
    Microchip Technology Inc.

  File Name:
    app.h

  Summary:
    This header file provides prototypes and definitions for the application.

  Description:
    This header file provides function prototypes and data type definitions for
    the application.  Some of these are required by the system (such as the
    "APP_Initialize" and "APP_Tasks" prototypes) and some of them are only used
    internally by the application (such as the "APP_STATES" definition).  Both
    are defined here for convenience.
 
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2017 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
//DOM-IGNORE-END

#ifndef _APP_H
#define _APP_H

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdlib.h>
#include "system_config.h"
#include "system_definitions.h"

// DOM-IGNORE-BEGIN
#ifdef __cplusplus  // Provide C++ Compatibility

extern "C" {

#endif
// DOM-IGNORE-END 

typedef enum
{
	/* Application's state machine's initial state. */
	APP_STATE_INIT=0,
    APP_STATE_WAIT_INIT,
    APP_STATE_IDLE,            
} APP_STATES;

typedef enum
{
	/* Buttons' state machine's initial state. */
    BUTTON_STATE_IDLE=0,
    BUTTON_STATE_PRESSED,      
    BUTTON_STATE_UPPAIRING_PRESSED,
    BUTTON_STATE_DOWNCONNECT_PRESSED,            
    BUTTON_STATE_NEXTFF_PRESSED,            
    BUTTON_STATE_PLAYPAUSE_PRESSED,
    BUTTON_STATE_PREVREWIND_PRESSED,            
    BUTTON_STATE_WAIT_FOR_RELEASE,            
    BUTTON_STATE_RELEASED,
} BUTTON_STATES;

#define BUTTON_DEBOUNCE       50        // 50 ms
#define LONG_BUTTON_PRESS   1000        // long button press is 1 sec

#define VOLUME_DELAY        1000        // 1 sec

// BT Audio DK Application LED's 
#define APP_LED1_ON()                                   LATGbits.LATG14 = 1;    //RG14(95)
#define APP_LED1_OFF()                                  LATGbits.LATG14 = 0;
#define APP_LED2_ON()                                   LATGbits.LATG13 = 1;    //RG13(97))
#define APP_LED2_OFF()                                  LATGbits.LATG13 = 0;
#define APP_LED3_ON()                                   LATCbits.LATC4 = 1;     //RC4(9)
#define APP_LED3_OFF()                                  LATCbits.LATC4 = 0;
#define APP_LED4_ON()                                   LATCbits.LATC3 = 1;     //RC3(8)
#define APP_LED4_OFF()                                  LATCbits.LATC3 = 0;
#define APP_LED5_ON()                                   LATGbits.LATG12 = 1;    //RG12(96)
#define APP_LED5_OFF()                                  LATGbits.LATG12 = 0;



//#define APP_LED1_ON()                                   BSP_LEDOn(BSP_LED_5)
//#define APP_LED1_OFF()                                  BSP_LEDOff(BSP_LED_5)
//#define APP_LED2_ON()                                   BSP_LEDOn(BSP_LED_6)
//#define APP_LED2_OFF()                                  BSP_LEDOff(BSP_LED_6)
//#define APP_LED3_ON()                                   BSP_LEDOn(BSP_LED_7)
//#define APP_LED3_OFF()                                  BSP_LEDOff(BSP_LED_7)
//#define APP_LED4_ON()                                   BSP_LEDOn(BSP_LED_8)
//#define APP_LED4_OFF()                                  BSP_LEDOff(BSP_LED_8)
//#define APP_LED5_ON()                                   BSP_LEDOn(BSP_LED_9)
//#define APP_LED5_OFF()                                  BSP_LEDOff(BSP_LED_9)


// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    Application strings and buffers are be defined outside this structure.
 */

typedef struct
{
    /* The application's current state */
    APP_STATES state;
    BUTTON_STATES buttonState;
    uint16_t buttonDelay;
    uint16_t volumeDelay;    
    uint8_t volume;     // 0-255 for DAC, adjust for AVRCP
    DRV_BM64_LINKSTATUS linkStatus;
} APP_DATA;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Routines
// *****************************************************************************
// *****************************************************************************
/* These routines are called by drivers when certain events occur.
*/
	
// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Summary:
     MPLAB Harmony application initialization routine.

  Description:
    This function initializes the Harmony application.  It places the 
    application in its initial state and prepares it to run so that its 
    APP_Tasks function can be called.

  Precondition:
    All other system initialization routines should be called before calling
    this routine (in "SYS_Initialize").

  Parameters:
    None.

  Returns:
    None.

  Example:
    <code>
    APP_Initialize();
    </code>

  Remarks:
    This routine must be called from the SYS_Initialize function.
*/

void APP_Initialize ( void );


/*******************************************************************************
  Function:
    void APP_Tasks ( void )

  Summary:
    MPLAB Harmony Demo application tasks function

  Description:
    This routine is the Harmony Demo application's tasks function.  It
    defines the application's state machine and core logic.

  Precondition:
    The system and application initialization ("SYS_Initialize") should be
    called before calling this.

  Parameters:
    None.

  Returns:
    None.

  Example:
    <code>
    APP_Tasks();
    </code>

  Remarks:
    This routine must be called from SYS_Tasks() routine.
 */

void APP_Tasks( void );
void buttonTasks( void );
void Timer_1ms(uintptr_t context, uint32_t currTick);
void syncLR();  // actually in audio.c, called from system_interrupt.c which includes app.h

#endif /* _APP_H */

//DOM-IGNORE-BEGIN
#ifdef __cplusplus
}
#endif
//DOM-IGNORE-END

/*******************************************************************************
 End of File
 */

